"""Tool that invokes the given command.

If the given command is given with a relative path, then this is
interpreted as a project local, and the failure is determined by the
``--if-no-op`` flag.

"""
import argparse
import logging

from menhir.tool import Tool
from menhir.tool_utils import OK, FAIL, NOTHING_TO_DO, tool_env, working_dir

log = logging.getLogger(__name__)


def tool():
    return Exec()


class Exec(Tool):

    def dir_info(tool, path, info):
        return {
            'project_recognised': False,
            'can_run': True,
        }

    def dependencies(tool, path):
        return []

    def arg_parser(tool, **kwargs):
        """Arg parser for the tool options and arguments."""
        return parser(**kwargs)

    def execute_tool(tool, path, info, args):
        """Execute the script tool."""
        import os
        from os.path import basename, isabs
        import subprocess

        with working_dir(path):
            command = args.command_name
            if basename(command) != command and not isabs(command):
                if (
                        not os.access(command, os.X_OK)
                ):
                    return NOTHING_TO_DO

            cmd = [command] + args.command_args
            log.info('Running "%s" in %s', " ".join(cmd), path)
            if 'changed' in info and not (
                    info['changed'].get('self') or
                    info['changed'].get('dependents')
            ):
                return OK

            env = tool_env()
            changed = info.get('changed')
            if changed is None:
                env['MENHIR_ALL'] = "1"
            else:
                if changed.get('self'):
                    env['MENHIR_CHANGED_SELF'] = "1"
                if changed.get('dependents'):
                    env['MENHIR_CHANGED_DEPENDENTS'] = "1"
                if changed.get('dependees'):
                    env['MENHIR_CHANGED_DEPENDEES'] = "1"

            result = subprocess.call(cmd, env=env)
            log.debug('Exec %s result: %s', args.command_name, result)
            if result:
                return FAIL
            return OK


def parser(**kwargs):
    """Arg parser for the tool options and arguments."""
    parser = argparse.ArgumentParser(
        description="Execute command.",
        **kwargs
    )
    parser.add_argument(
        'command_name',
        metavar='command-name',
        help="""Command to invoke from bin directory.

If this includes a relative path it is interpreted as being project
local, and if it doesn't exist, then failure failure is determined by
the ``--if-no-op`` flag """
    )
    parser.add_argument(
        'command_args',
        nargs=argparse.REMAINDER,
        help='command arguments'
    )
    return parser



try:
    from functools import lru_cache
except:
    from functools32 import lru_cache


@lru_cache()
def sha():
    from menhir.gitutils import head_commit, sha
    return sha(head_commit(repo()))


@lru_cache()
def branch():
    from menhir.gitutils import branch
    return branch(repo())


@lru_cache()
def repo():
    from menhir.gitutils import repo
    return repo()


def image(info, path):
    repo = info.get('docker', {}).get('repository')
    if repo:
        return '%s/%s:%s' % (
            repo,
            info['project-name'],
            sha()
        )



"""
Pyenv based tool for menhir.

The ``pyenv`` tool provides the ``virtualenv`` target to create a
virtualenv for each of the repository's projects.  The virtualenv is
loaded with all the project's dependencies as specified in
``requirements*.txt``.
"""
from __future__ import absolute_import

import argparse
import logging
import subprocess

from os import getenv
from os.path import expanduser, isdir, join

from git import Repo
from menhir.tool import Tool
from menhir.tool_utils import (
    FAIL,
    OK,
    package_script,
    tool_env,
    working_dir
)

from menhir.utils import method, multi

log = logging.getLogger(__name__)


def tool():
    return Pyenv()


class Pyenv(Tool):

    def dir_info(tool, path, info):
        from glob import glob
        from os.path import exists, join
        setup_py = join(path, 'setup.py')
        requirements = join(path, 'requirements*.txt')
        files = glob(requirements)
        has_requirements = exists(setup_py) and bool(files)
        return {
            'project_recognised': has_requirements,
            'can_run': has_requirements,
        }

    def dependencies(tool, path):
        return []

    def arg_parser(tool, **kwargs):
        return parser(**kwargs)

    def execute_tool(tool, path, info, args,):
        """Execute a build phase."""
        return task(path, info, args)


def parser(**kwargs):
    parser = argparse.ArgumentParser(
        description="Manage project specific pyenv virtualenvs",
        **kwargs
    )
    parsers = parser.add_subparsers(help="pyenv commands", dest='phase')
    p = parsers.add_parser(
        'virtualenv',
        help='Build a pyenv virtualenv with the projects dependencies'
    )
    p.add_argument('remainder', nargs=argparse.REMAINDER)

    p = parsers.add_parser(
        'install-virtualenv',
        help='Install pyenv-virtualenv (assumes pyenv installed)'
    )
    p.add_argument('remainder', nargs=argparse.REMAINDER)

    return parser


@multi
def task(path, info, args):
    return args.phase


@method(task, 'virtualenv')
def task_virtualenv(path, info, args):
    if (
            'changed' not in info or
            info['changed'].get('self') or
            info['changed'].get('dependents')
    ):
        log.info('Running pyenv requirements in %s', path)

        env = tool_env()

        with package_script("/tools/pyenv/requirements.sh") as f:
            with working_dir(path):
                res = subprocess.call(
                    [f.name, path, info['project-name']],
                    env=env,
                )
                if res:
                    return FAIL
                return OK
    else:
        log.info('not running pyenv requirements in %s', path)
        return OK


PYENV_VENV_URL = 'https://github.com/yyuu/pyenv-virtualenv.git'


@method(task, 'install-virtualenv')
def task_install_virtualenv(path, info, args):
    pyenv_root = getenv('PYENV_ROOT') or expanduser("~/.pyenv")
    dest = join(pyenv_root, "plugins/pyenv-virtualenv")
    if not isdir(dest):
        Repo.clone_from(PYENV_VENV_URL, dest)
        with open(expanduser("~/.bash_profile"), 'a') as f:
            f.write('eval "$(pyenv virtualenv-init -)"\n')
    return OK
